/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "array.h"

#ifdef ARRAY_USE_GLIB

void array_set(Array* a,int pos,void* data){
    if(pos>=a->len)
        g_ptr_array_set_size(a,pos+1);
    a->pdata[pos] = data;
}

#else

#include <string.h>                     /* memmove() */

#include "array.h"
#include "memutil.h"


Array* array_init(Array* a){
    a->size = 0;
    allocation_init(&a->alloc,1);
    return a;
}


void array_done(Array* a){
    allocation_done(&a->alloc);
}


Array* array_new(int size){
    return array_init(u_new(Array,1));
}


void array_free(Array* a){
    array_done(a);
    u_free(a);
}


#if 0
static void array_resize_allocation(Array* a,int size){
    if(size>0){
        if(a->allocation_size>=size)
            return;
        a->arr = u_renew(void*,a->arr,size);
        a->length = a->allocation_size = size;
    }else{
        /* don't care */
    }
}
#endif


static int array_ensure_size(Array* a,int size){
    if(size>a->size){
        a->arr = (void**)allocation_ensure_full(&a->alloc,size*sizeof(void*),(const char*)a->arr,a->size*sizeof(void*));
        a->size = size;
    }
    return a->alloc.size/sizeof(void*);
}


void array_resize(Array* a,size_t size){
    if(size>a->size)
        a->arr = (void**)allocation_ensure_full(&a->alloc,size*sizeof(void*),(const char*)a->arr,a->size*sizeof(void*));
    a->size = size;
}


void array_set(Array* a,int pos,void* data){
    if(pos>=a->size){
        array_ensure_size(a,pos+1);
        a->size = pos+1;
    }
    a->arr[pos] = data;
}


void array_insert(Array* a,int pos,void* data){
    array_ensure_size(a,a->size+1);
    if(pos<a->size-1)
        memmove(&a->arr[pos+1],&a->arr[pos],(a->size-pos-1)*sizeof(void*));
    a->arr[pos] = data;
}


void* array_remove(Array* a,int pos){
    void* data = a->arr[pos];
    --a->size;
    if(pos<a->size)
        memmove(&a->arr[pos],&a->arr[pos+1],(a->size-pos)*sizeof(void*));
    return data;
}


void* array_pop(Array* a){
    return a->arr[--a->size];
}

#endif /* ! ARRAY_USE_GLIB */

void array_free_full(Array* a,ArrayNotifyFunc func){
    int i;
    for(i=0;i<array_size(a);++i)
        func(array_get(a,i));
    array_free(a);
}


int array_foreach(Array* a,ArrayForeachFunc func,void* userdata){
    int i;
    for(i=0;i<array_size(a);++i){
        int result = func(array_get(a,i),userdata);
        if(result)
            return result;
    }
    return 0;
}
