/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef array_h
#define array_h

#ifdef USE_GLIB
//#define ARRAY_USE_GLIB
#endif

#ifdef ARRAY_USE_GLIB

#include <glib.h>

#define Array               GPtrArray
#define array_new(a)        g_ptr_array_sized_new(a)
#define array_free(a)       g_ptr_array_unref(a)
#define array_size(a)       (a)->len
#define array_resize(a,b)   g_ptr_array_set_size(a,b)
void array_set(Array*,int pos,void* data);
#define array_get(a,b)      (a)->pdata[b]
#define array_iter(a,b)     &(a)->pdata[b]
#define array_insert(a,b,c) g_ptr_array_insert(a,b,c)
#define array_prepend(a,b)  g_ptr_array_insert(a,0,b)
#define array_append(a,b)   g_ptr_array_add(a,b)
#define array_remove(a,b)   g_ptr_array_remove_index(a,b)
#define array_pop(a)        g_ptr_array_remove_index_fast(a,(a)->len-1)
#define array_insert_range(a,b,c) g_ptr_array_insert_range(a,b,c)
#define array_remove_range(a,b,c) g_ptr_array_remove_range(a,b,c)

#else

#include "allocation.h"

typedef struct{
    void** arr;
    size_t size;
    Allocation alloc;
}Array;

Array* array_init(Array*);
void array_done(Array*);
Array* array_new(int size);
void array_free(Array*);
#define array_size(a) (a)->size
void array_resize(Array*,size_t);
void array_set(Array*,int pos,void* data);
#define array_get(a,b) (a)->arr[b]
#define array_iter(a,b) &(a)->arr[b]
void array_insert(Array*,int pos,void* data);
#define array_prepend(a,b) array_insert(a,0,b)
#define array_append(a,b) array_set(a,array_size(a),b)
void* array_remove(Array*,int pos);
void* array_pop(Array*);
void array_insert_range(Array*,int pos,void**,size_t);
void array_remove_range(Array*,int pos,size_t);

#endif

typedef void(*ArrayNotifyFunc)(void*);
typedef int(*ArrayForeachFunc)(void*,void*);

#define array_push(a,b) array_append(a,b)
void array_free_full(Array*,ArrayNotifyFunc);
int array_foreach(Array*,ArrayForeachFunc,void*);

#endif /* array_h */
