/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <extractor.h>
#include <sys/stat.h>

#include "debug.h"
#include "memutil.h"

#define DATA_WIDTH 1
#define DATA_HEIGHT 2
#define DATA_MIMETYPE 4


int file_exists(const char* filename){
    struct stat buf;
    return stat(filename,&buf)==0;
}


typedef struct{
    int width;
    int height;
    char* mimetype;
    int need_data;
}ImageinfoData;
static int get_imageinfo_cb(
    void* userdata,
    const char* pluginname,
    enum EXTRACTOR_MetaType type,
    enum EXTRACTOR_MetaFormat format,
    const char* data_mime_type,
    const char* data,
    size_t data_len
){
    ImageinfoData* d = (ImageinfoData*)userdata;
    switch(type){
    case EXTRACTOR_METATYPE_IMAGE_DIMENSIONS:
        if(d->need_data&(DATA_WIDTH|DATA_HEIGHT)){
            sscanf(data,"%dx%d",&d->width,&d->height);
            d->need_data &= ~(DATA_WIDTH|DATA_HEIGHT);
        }
        break;
    case EXTRACTOR_METATYPE_MIMETYPE:
        if(d->need_data&DATA_MIMETYPE){
            d->mimetype = u_strndup(data,data_len);
            d->need_data &= ~DATA_MIMETYPE;
        }
        break;
    default:
        break;
    }
    return !d->need_data;
}
int get_imageinfo(const char* filename,int* x_ret,int* y_ret,char** mimetype_ret){
    static struct EXTRACTOR_PluginList* el;
    ImageinfoData d;
    if(!file_exists(filename))
        return 0;
    d.mimetype = NULL;
    d.need_data = (x_ret?DATA_WIDTH:0)|(y_ret?DATA_HEIGHT:0)|(mimetype_ret?DATA_MIMETYPE:0);
    if(!d.need_data)
        return 1;
    if(!el){
#if 0
        //el = EXTRACTOR_plugin_add_defaults(EXTRACTOR_OPTION_DEFAULT_POLICY|EXTRACTOR_OPTION_IN_PROCESS);
#else
        el  = EXTRACTOR_plugin_add(el,"gif",NULL,EXTRACTOR_OPTION_IN_PROCESS);
        el  = EXTRACTOR_plugin_add(el,"jpeg",NULL,EXTRACTOR_OPTION_IN_PROCESS);
        el  = EXTRACTOR_plugin_add(el,"png",NULL,EXTRACTOR_OPTION_IN_PROCESS);
#endif
    }
    EXTRACTOR_extract(el,filename,NULL,0,get_imageinfo_cb,&d);
    //EXTRACTOR_plugin_remove_all(el);
    if(x_ret)
        *x_ret = d.width;
    if(y_ret)
        *y_ret = d.height;
    if(mimetype_ret){
        *mimetype_ret = d.mimetype;
    }else if(d.mimetype){
        u_free(d.mimetype);
    }
    return !d.need_data;
}
