/*
 * Copyright (C) 2009 Mica, Östersunds kummun
 * Copyright (C) 2016 Magnus Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#ifndef hash_h
#define hash_h

#ifdef USE_GLIB
//# define HASH_USE_GLIB
#endif

#ifdef HASH_USE_GLIB

#include <glib.h>

#define HashMap GHashTable
#define EqualFunc GEqualFunc
#define HashFunc GHashFunc
#define HashMapNotifyFunc GDestroyNotify
#define HashMapForeachFunc GHFunc

#define hash_map_new(a,b,c,d) g_hash_table_new_full(a,b,c,d)
#define hash_map_free(a) g_hash_table_destroy(a)
#define hash_map_add(a,b,c) g_hash_table_insert(a,b,c)
#define hash_map_get(a,b) g_hash_table_lookup(a,b)
#define hash_map_remove(a,b) g_hash_table_remove(a,b)
#define hash_map_remove_all(a) g_hash_table_remove_all(a)
#define hash_map_size(a) g_hash_table_size(a)
#define hash_map_foreach(a,b,c) g_hash_table_foreach(a,b,c)

/* Hash Functions */
#define direct_equal g_direct_equal
#define direct_hash g_direct_hash
#define int_equal g_int_equal
#define int_hash g_int_hash
#define str_equal g_str_equal
#define str_hash g_str_hash

#else /* HASH_USE_GLIB */

typedef struct _HashMap HashMap;
typedef int(*EqualFunc)(const void*,const void*);
typedef unsigned int(*HashFunc)(const void*);
typedef void(*HashMapNotifyFunc)(void*);
typedef int(*HashMapForeachFunc)(void* key,void* value,void* userdata);

HashMap* hash_map_new(HashFunc,EqualFunc,HashMapNotifyFunc notify_key,HashMapNotifyFunc notify_value);
void  hash_map_free(HashMap*);
void  hash_map_add(HashMap*,void* key,void* value);
void* hash_map_get(HashMap*,const void* key);
int hash_map_remove(HashMap*,void* key);
void hash_map_remove_all(HashMap*);
int hash_map_size(HashMap*);
int hash_map_foreach(HashMap*,HashMapForeachFunc,void*);

/* Hash Functions */
unsigned int hash_combine(unsigned int,unsigned int);
int direct_equal(const void*,const void*);
unsigned int direct_hash(const void*);
#define pointer_equal(a,b) direct_equal(a,b)
unsigned int pointer_hash(const void*);
int int_equal(const void*,const void*);
unsigned int int_hash(const void*);
int str_equal (const void*,const void*);
unsigned int str_hash_update(unsigned int,const char* s,int len);
unsigned int str_hash(const void*);

#endif /* HASH_USE_GLIB */

#endif /* hash_h */
