/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef listmodel_h
#define listmodel_h

#include "queue.h"
#include "signals.h"
#include "uref.h"


typedef void* ModelIter;

typedef void(*ModelItemNotifyFunc)(void*);
typedef int(*ModelFilterFunc)(const void*);
typedef int(*ModelCompareFunc)(const void*,const void*);
typedef int(*ModelSignalHandler)(void* userdata,void* data,int num);
typedef int(*ModelForeachFunc)(void* userdata,void* data,int num,ModelIter*);

int model_signal_invoke(Signal*,void*,int);


typedef struct _ListModel ListModel;

typedef struct{
    void(*done)(ListModel*);
    int(*length)(ListModel*);
    ModelIter*(*first)(ListModel*);
    ModelIter*(*last)(ListModel*);
    ModelIter*(*previous)(ListModel*,ModelIter*);
    ModelIter*(*next)(ListModel*,ModelIter*);
    int(*compare)(ListModel*,ModelIter*,ModelIter*);
    int(*foreach)(ListModel*,ModelForeachFunc,void* userdata);
    ModelIter*(*get_iter)(ListModel*,int num);
    int(*get_num)(ListModel*,ModelIter*);
    void*(*get)(ListModel*,ModelIter*);
}ListModelInterface;

struct _ListModel{
    URef ref;
    const ListModelInterface* vt;
    Signal signal_add;
    Signal signal_remove;
};

void list_model_done_func(ListModel*);

ListModel* list_model_init(ListModel*);
#if 1
# define list_model_ref(a) (ListModel*)u_ref_ref((URef*)a)
# define list_model_unref(a) u_ref_unref((URef*)a)
#else
ListModel* list_model_ref(ListModel*);
void list_model_unref(ListModel*);
#endif
#define list_model_length(a) (a)->vt->length(a)
#define list_model_first(a) (a)->vt->first(a)
#define list_model_last(a) (a)->vt->last(a)
#define list_model_previous(a,b) (a)->vt->previous(a,b)
#define list_model_next(a,b) (a)->vt->next(a,b)
#define list_model_compare(a,b,c) (a)->vt->compare(a,b,c)
#define list_model_foreach(a,b,c) (a)->vt->foreach(a,b,c)
#define list_model_get_iter(a,b) (a)->vt->get_iter(a,b)
#define list_model_get_num(a,b) (a)->vt->get_num(a,b)
#define list_model_get(a,b) (a)->vt->get(a,b)

SignalSubscription* list_model_subscribe_add(ListModel*,ModelSignalHandler,void* userdata);
#define list_model_unsubscribe_add(a,b) signal_unsubscribe(&(a)->signal_add,b)
SignalSubscription* list_model_subscribe_remove(ListModel*,ModelSignalHandler,void* userdata);
#define list_model_unsubscribe_remove(a,b) signal_unsubscribe(&(a)->signal_remove,b)


typedef struct{
    //void* data;
    ListModel* model;
    int num;
    ModelIter* iter;
    SignalSubscription* ss_add;
    SignalSubscription* ss_remove;
}ListModelRow;

ListModelRow* list_model_row_init(ListModelRow*,ListModel*);
void list_model_row_done(ListModelRow*);
ListModelRow* list_model_row_new(ListModel*);
void list_model_row_free(ListModelRow*);
void list_model_row_set(ListModelRow*,ModelIter*);
ModelIter* list_model_row_iter(ListModelRow*);
void* list_model_row_data(ListModelRow*);
void list_model_row_first(ListModelRow*);
void list_model_row_last(ListModelRow*);
int list_model_row_previous(ListModelRow*);
int list_model_row_next(ListModelRow*);
#define list_model_row_valid(a) ((a)->num>=0)


ListModel* list_model_store_new(ModelItemNotifyFunc);

void list_model_store_prepend(ListModel*,void*);
void list_model_store_append(ListModel*,void*);
void list_model_store_insert_before(ListModel*,ModelIter*,void*);
void list_model_store_insert_after(ListModel*,ModelIter*,void*);
void* list_model_store_remove(ListModel*,ModelIter*);
void list_model_store_remove_data(ListModel*,void*);

#endif /* model_h */
