/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef memchunk_h
#define memchunk_h

#include <sys/types.h>

#ifdef USE_REFCOUNT
#include "uref.h"
#endif

#define MEMCHUNK_GREEDY   (1<<0)
#define MEMCHUNK_WRITABLE (1<<1)
#define MEMCHUNK_COPY     (1<<2)
#define MEMCHUNK_STEAL    (1<<3)
#define MEMCHUNK_NOTIFY   (1<<4)

typedef void(*MemchunkNotify)(void*);

typedef struct{
#ifdef USE_REFCOUNT
    URef ref;
#endif
    size_t size;
    /* only valid if size > 0 */
    char* mem;
    /* only valid if private = 0 */
    struct{
        MemchunkNotify func;
        void* data;
    }notify;
    unsigned int greedy:1;
    unsigned int is_private:1;
    unsigned int is_writable:1;
}Memchunk;

Memchunk* memchunk_init(Memchunk*,int flags);
void memchunk_done(Memchunk*);
Memchunk* memchunk_new(int flags);
#ifdef USE_REFCOUNT
# define memchunk_ref(a) (Memchunk*)u_ref_ref((URef*)a)
# define memchunk_unref(a) u_ref_unref((URef*)a)
#else
void memchunk_free(Memchunk*);
#endif

char* memchunk_ensure_private(Memchunk*);
char* memchunk_ensure_private_size_nopreserve(Memchunk*,size_t);
char* memchunk_ensure_writable(Memchunk*);
char* memchunk_ensure_writable_size_nopreserve(Memchunk*,size_t);
int memchunk_try_ensure_size(Memchunk*,size_t);
char* memchunk_ensure_size_full(Memchunk*,size_t,const char* preserve_begin,size_t preserve_size);
#define memchunk_ensure_size_nopreserve(a,b) memchunk_ensure_size_full(a,b,NULL,0)

Memchunk* memchunk_set(Memchunk*,const char* mem,size_t size,int flags,...);

#endif /* memchunk_h */
