/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "priv.h"

#include "memutil.h"
#include "namespace.h"

#define N_NUMBER 10
#define N_LOWER 26
#define N_OTHER 1
#define N_UPPER 26
#define N_FIRST (N_OTHER+N_UPPER)
#define N_TOTAL (N_NUMBER+N_LOWER+N_FIRST)

#define NS_BIGENDIAN

NameSpace* ns_init(NameSpace* ns){
    ns->counter = 1;
    return ns;
}


void ns_done(NameSpace* ns){
}


char* ns_identifier(NameSpace* ns){
    const char ch[] = "0123456789"
                      "abcdefghij"
                      "klmnopqrst"
                      "uvwxyz"
                      "_"
                      "ABCDEFGHIJ"
                      "KLMNOPQRST"
                      "UVWXYZ";
    char* s;
    int c = ns->counter++;

    if(c<N_FIRST){
        s = u_new(char,2);
        s[0] = ch[(N_TOTAL-N_FIRST)+c];
        s[1] = '\0';
        return s;
    }
    c -= N_FIRST;
    if(c<N_FIRST*N_TOTAL){
        s = u_new(char,3);
#ifdef NS_BIGENDIAN
        s[0] = ch[(N_TOTAL-N_FIRST)+c/N_TOTAL];
        s[1] = ch[c%N_TOTAL];
#else
        s[0] = ch[(N_TOTAL-N_FIRST)+c%N_FIRST];
        s[1] = ch[c/N_FIRST];
#endif
        s[2] = '\0';
        return s;
    }
    c -= N_FIRST*N_TOTAL;
    if(c<N_FIRST*N_TOTAL*N_TOTAL){
        s = u_new(char,4);
#ifdef NS_BIGENDIAN
        s[0] = ch[(N_TOTAL-N_FIRST)+c/(N_TOTAL*N_TOTAL)];
        s[1] = ch[(c/N_TOTAL)%N_TOTAL];
        s[2] = ch[c%N_TOTAL];
#else
        s[0] = ch[(N_TOTAL-N_FIRST)+c%N_FIRST];
        s[1] = ch[(c/N_FIRST)%N_TOTAL];
        s[2] = ch[c/(N_FIRST*N_TOTAL)];
#endif
        s[3] = '\0';
        return s;
    }
    c -= N_FIRST*N_TOTAL*N_TOTAL;
    if(c<N_FIRST*N_TOTAL*N_TOTAL*N_TOTAL){
        s = u_new(char,5);
#ifdef NS_BIGENDIAN
        s[0] = ch[(N_TOTAL-N_FIRST)+c/(N_TOTAL*N_TOTAL*N_TOTAL)];
        s[1] = ch[(c/(N_TOTAL*N_TOTAL))%N_TOTAL];
        s[2] = ch[(c/N_TOTAL)%N_TOTAL];
        s[3] = ch[c%N_TOTAL];
#else
        s[0] = ch[(N_TOTAL-N_FIRST)+c%N_FIRST];
        s[1] = ch[(c/N_FIRST)%(N_TOTAL*N_TOTAL)];
        s[2] = ch[(c/(N_FIRST*N_TOTAL))*N_TOTAL];
        s[3] = ch[c/(N_FIRST*N_TOTAL*N_TOTAL)];
#endif
        s[4] = '\0';
        return s;
    }
    /* Give up on > 6860160 identifiers */
    return NULL;
}


/*
const char* ns_get(NameSpace* ns,void* p){
    return (const char*)hash_map_get(ns->name_map,p);
}


const char* ns_add(NameSpace* ns,void* p){
    char* name = ns_identifier(ns);
    hash_map_add(ns->name_map,p,name);
    return name;
}
*/
