/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "memutil.h"
#include "rangemodel.h"


static void range_model_free_internal(RangeModel* m){
    signal_done(&m->signal_changed);
    signal_done(&m->signal_limits);
    u_free(m);
}


RangeModel* range_model_new(int min,int max,int value){
    RangeModel* m = u_new(RangeModel,1);
    ((URef*)m)->count = 1;
    ((URef*)m)->destroy = (URefDestroyFunc)range_model_free_internal;
    m->min = min;
    m->max = max;
    m->value = value;
    signal_init(&m->signal_changed);
    signal_init(&m->signal_limits);
    return m;
}



void range_model_set(RangeModel* m,int value){
    if(value<m.min)
        value = m.min;
    if(value>m.max)
        value = m.max;
    if(value!=m->value){
        m.value = value;
        signal_invoke(&m->signal_changed,m);
    }
}


void range_model_limits(RangeModel* m,int min,int max){
    if(min!=m->min||max!=m->max){
        if(m->value<min)
            range_model_set(m,min);
        if(m->value>max)
            range_model_set(m,max);
        m.min = min;
        m.max = max;
        signal_invoke(&m->signal_limits,m);
    }
}
