/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef session_h
#define session_h

#include "config.h"

#include <stdio.h>

#ifdef USE_MICROHTTPD
# include <microhttpd.h>
#else
# include "httpd.h"
#endif

#include "namespace.h"
#include "queue.h"
#include "strn.h"
#include "widgets/widget.h"


#define SESSION_COOKIE_NAME "session"

/**
  One connection to the client.
*/
typedef struct{
#ifdef USE_MICROHTTPD
    int pipe_fd[2];
#else
    HttpdConnection* conn;
    int bytes;
#endif
}SessionConnection;

SessionConnection* session_connection_new();
void session_connection_free(SessionConnection*);


typedef struct _UpSession Session;
struct _UpSession{
    char* id;
    Strn buffer;
    Queue* connection_queue;
    SessionConnection* connection;
    NameSpace jsid_ns;
    HashMap* jsid_map;
    NameSpace callback_ns;
    HashMap* callback2name;
    HashMap* name2callback;
    UpWidget* root;
};


typedef enum{
    SC_JS,
    SC_DATA,
    SC_FILE
}SessionCallbackType;
typedef void(*SessionJsFunc)(void* userdata,Session*,const char*);
typedef void(*SessionDataFunc)(void* userdata,Strn*,char** mimetype);
typedef void(*SessionFileFunc)(void* userdata,char** filename,char** mimetype);
typedef struct{
    SessionCallbackType type;
    void* data;
    union{
        SessionJsFunc js_func;
        SessionDataFunc data_func;
        SessionFileFunc file_func;
    };
}SessionCallback;



//Session* session_init(FILE* fd);

Session* session_new();
void session_free(Session*);
Session* session_get(const char*);
void session_add(Session*);

#ifdef USE_MICROHTTPD
SessionConnection* session_open_connection(Session*,int include_session_cookie);
#else
SessionConnection* session_open_connection(Session*,HttpdConnection*,int include_session_cookie);
#endif
int session_close_connection(Session*);
int session_try_close_connection(Session*);
int session_bytes_for_connection(Session*);
int session_send(Session*,const char*,int);
int session_sendf(Session*,const char* format,...);
int session_sendvf(Session*,const char* format,va_list);

const char* session_get_jsid(Session*,void*);
const char* session_add_jsid(Session*,void*);
const char* session_register_js_callback(Session*,SessionJsFunc,void*);
const char* session_register_data_callback(Session*,SessionDataFunc,void*);
const char* session_register_file_callback(Session*,SessionFileFunc,void*);
//#define session_register_js_callback(a,b,c) session_register_callback(a,SC_JS,b,c)
//#define session_register_data_callback(a,b,c) session_register_callback(a,SC_DATA,b,c)
SessionCallback* session_get_callback(Session*,const char* name);
int session_do_callback(Session*,const char* name,const char* args);
char* session_get_buffer(Session*,size_t* size);

#endif
