/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef signals_h
#define signals_h

#include "list.h"

typedef int(*SignalHandler)(void* userdata,void* signaldata);
typedef int(*SignalMarshall)(SignalHandler,void* userdata,void* signaldata);
#ifdef USE_REFCOUNT
typedef void(*SignalUserdataDestroy)(void*);
#endif


typedef struct _SignalSubscription SignalSubscription;
struct _SignalSubscription{
    SignalHandler handler;
    void* userdata;
#ifdef USE_REFCOUNT
    SignalUserdataDestroy destroy;
#endif
};

#define signal_subscription_get_userdata(s) ((s)->userdata)

typedef struct _Signal Signal;
struct _Signal{
    List* subscription_list;
};

Signal* signal_init(Signal*);
void signal_done(Signal*);
Signal* signal_new();
void signal_free(Signal*);
#ifdef USE_REFCOUNT
SignalSubscription* signal_subscribe_full(Signal*,SignalHandler,void* userdata,SignalUserdataDestroy);
#define signal_subscribe(a,b,c) signal_subscribe_full(a,b,c,NULL)
SignalSubscription* signal_subscribe_last_full(Signal*,SignalHandler,void* userdata,SignalUserdataDestroy);
#define signal_subscribe_last(a,b,c) signal_subscribe_last_full(a,b,c,NULL)
void signal_unsubscribe(Signal*,SignalSubscription*);
#else
SignalSubscription* signal_subscribe(Signal*,SignalHandler,void* userdata);
SignalSubscription* signal_subscribe_last(Signal*,SignalHandler,void* userdata);
void* signal_unsubscribe(Signal*,SignalSubscription*);
#endif
int signal_invoke(Signal*,void*);
int signal_invoke_marshal(Signal*,SignalMarshall,void*);

#endif /* signals_h */
