/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef strn_h
#define strn_h

#include <stdarg.h>
#include <stddef.h>
#include <unistd.h>                     /* ssize_t */

#include "memchunk.h"
#include "memutil.h"
#ifdef USE_REFCOUNT
# include "uref.h"
#endif

#define TEST_MEMCHUNK

#ifdef TEST_MEMCHUNK
typedef MemchunkNotify StrnNotifyFunc;
#else
typedef void(*StrnNotifyFunc)(void*);
#endif

typedef struct{
#ifdef TEST_MEMCHUNK
    Memchunk chunk;
#else
# ifdef USE_REFCOUNT
    URef ref;
# endif
    size_t allocation;
    unsigned int greedy:1;
    //unsigned int private_allocation:1;
#endif
    char* str;
    size_t len;
    unsigned int zero_terminated:1;
}Strn;

Strn* strn_init(Strn*,int greedy);
void strn_done(Strn*);
Strn* strn_new(int greedy);
#ifdef USE_REFCOUNT
# define strn_ref(a) (Strn*)u_ref_ref((URef*)a)
# define strn_unref(a) u_ref_unref((URef*)a)
//Strn* strn_ref(Strn*);
//void strn_unref(Strn*);
#else
void strn_free(Strn*);
#endif

#define strn_length(a) ((a)->len)
#define strn_reset(a) (a)->len=0
#ifdef TEST_MEMCHUNK
# define strn_is_writable(a) ((a)->chunk.writable)
# define strn_is_private(a) ((a)->chunk.private)
# define strn_is_const(a) (!strn_is_writable(a))
#else
# define strn_is_private(a) ((a)->allocation)
# define strn_is_const(a) (!strn_is_private(a))
#endif

void strn_ensure_private(Strn*);
void strn_ensure_writable(Strn*);
size_t strn_ensure_private_length(Strn*,size_t,int preserve);
size_t strn_ensure_writable_length(Strn*,size_t,int preserve);
const char* strn_ensure_zero_terminated(Strn*);

Strn* strn_set_full(Strn*,const char* s,ssize_t len,int copy);
#define strn_set_const(a,b,c) strn_set_full(a,b,c,0)
#define strn_set_dup(a,b,c) strn_set_full(a,b,c,1)
Strn* strn_set_steal(Strn*,char* s,ssize_t len);
#define strn_set_strn(a,b) strn_set_sub_strn(a,b,0,-1)
int strn_setf(Strn*,const char*,...);
int strn_setvf(Strn*,const char*,va_list);
Strn* strn_set_notify(Strn*,const char*,ssize_t,StrnNotifyFunc,void* data);
Strn* strn_set_sub(Strn*,const char*,off_t,ssize_t);
Strn* strn_set_sub_strn(Strn*,Strn*,off_t,ssize_t);

Strn* strn_append_full(Strn* sn,const char* s,ssize_t len,int copy);
#define strn_append(a,b) strn_append_full(a,b,-1,1)
#define strn_appendn(a,b,c) strn_append_full(a,b,c,1)
Strn* strn_append_steal(Strn*,char*,ssize_t);
int strn_appendf(Strn*,const char*,...);
int strn_appendvf(Strn*,const char*,va_list);
#define strn_append_strn(a,b,c) strn_append_full(a,(b)->str,(b)->len,c)
Strn* strn_append_char(Strn*,int);

void strn_trim(Strn*);
char* strn_dup(Strn*);
char* strn_steal(Strn*);
char* strn_copy_to(Strn*,char* dst);
char* strn_maybe_dup(Strn*,int* copied);
#define strn_asciiz(a) strn_ensure_zero_terminated(a)
#define strn_toi(a) ((a)->len>0?atoin((a)->str,(a)->len):0)

#define strn_cmp(a,b) strncmp(b,(a)->str,(a)->len)
#define strn_casecmp(a,b) strncasecmp(b,(a)->str,(a)->len)
int strn_equal(const void*,const void*);
unsigned int strn_hash(const void*);

#endif /* strn_h */
