/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <ctype.h>                      /* isblank() */
#define _GNU_SOURCE                     /* vasprintf() */
#include <stdio.h>                      /* snprintf() */
#include <string.h>

#include "debug.h"
#include "error.h"
#include "memutil.h"
#include "strutil.h"


#ifndef USE_GLIB
int vasprintf(char**,const char*,va_list);


char* u_strdupf(const char* format,...){
    char* s;
    va_list ap;
    va_start(ap,format);
#if 1
    vasprintf(&s,format,ap);
#else
    s = u_strdupvf(format,ap);
#endif
    va_end(ap);
    return s;
}


char* u_strdupvf(const char* format,va_list ap){
    char* s;
#if 0
    va_list ap2;
    va_copy(ap2,ap);
    int len = snprintf(NULL,0,format,ap);
    s = u_new(char,len+1);
    vsnprintf(s,len+1,format,ap2);
    va_end(ap2);
#else
    vasprintf(&s,format,ap);
#endif
    return s;
}
#endif


char* u_strset_steal(char** dst,char* src){
    if(*dst)
        u_free(*dst);
    return *dst = src;
}



/**
  Remove all occurances of the characters in the string. The string is
  modified in place.
*/
char* string_remove(char* string,char* characters){
    char* src;
    char* dst;
    for(src=dst=string;*src;++src){
        if(!strchr(characters,*src))
            *(dst++) = *src;
    }
    *dst = '\0';
    return string;
}


const char* string_replace(char* string,char* old,char* new){
    size_t len;
    char *p;

    len = strlen(new);
#ifdef DEBUG
    if(len>strlen(old)){
        DEBUG_p("the replacement string is longer than old one");
        return NULL;
    }
#endif
    for(p=string;*p;p+=len){
        p = strstr(p,old);
        if(!p)
            break;
        memcpy(p,new,len);
    }
    return string;
}

char* string_rtrim(char* s){
    char* p;
    p = strchr(s,'\0')-1;
    while(isblank(*p) && p>=s)
        *p-- = '\0';
    return s;
}



#ifdef USE_GLIB
int u_strnatcmp(const char* s1,const char* s2){
#ifdef USE_GLIB
    char* n1 = g_utf8_collate_key_for_filename(s1,-1);
    char* n2 = g_utf8_collate_key_for_filename(s2,-1);
    int result = strcoll(n1,n2);
    g_free(n1);
    g_free(n2);
    return result;
#else
    const char* p1;
    const char* p2;
    enum{ALPH,NUM} state = 0;
    for(;;){
        switch(state){
        case ALPH:
            /* FIXME!! */
            return strcoll(s1,s2);
        case NUM:
            /* FIXME!! */
            break;
        }
    }
#endif
}
#endif
