/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#include <libexif/exif-loader.h>

#include "debug.h"
#include "memutil.h"


static int get_imageinfo(const char* filename,int* x_ret,int* y_ret,char** mimetype_ret){
    ExifLoader* l = exif_loader_new();
    ExifData* ed;
    int rc;
    if(!l)
        return 0;

    /* Load the EXIF data from the image file */
    exif_loader_write_file(l,filename);

    /* Get a pointer to the EXIF data */
    ed = exif_loader_get_data(l);

    /* The loader is no longer needed--free it */
    exif_loader_unref(l);
    if(!ed)
        return 0;

    /* Make sure the image had a thumbnail before trying to write it */
    if (ed->data && ed->size) {
#if 0
        FILE *thumb;
        char thumb_name[1024];

        /* Try to create a unique name for the thumbnail file */
        snprintf(thumb_name, sizeof(thumb_name),"%s_thumb.jpg", argv[1]);
        thumb = fopen(thumb_name, "wb");
        if(thumb){
            /* Write the thumbnail image to the file */
            fwrite(ed->data, 1, ed->size, thumb);
            fclose(thumb);
            printf("Wrote thumbnail to %s\n", thumb_name);
            rc = 0;
        }else{
            printf("Could not create file %s\n", thumb_name);
            rc = 2;
        }
#endif
    }else{
        DEBUG_p("No EXIF thumbnail in file %s\n",filename);
        rc = 1;
    }
    /* Free the EXIF data */
    exif_data_unref(ed);
}


int main(int argc,char** argv){
    int x;
    int y;
    char* mimetype;
    if(!get_imageinfo("kirby.gif",&x,&y,&mimetype))
        return 1;
    DEBUG_p("x: %d, y: %d, mime-type: %s",x,y,mimetype);
    u_free(mimetype);
    return 0;
}
