/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "list.h"
#include "memutil.h"
#include "up.h"


char* register_identifier(){
    const char ch[] = "0123456789"
                      "_ABCDEFGHI"
                      "JKLMNOPQRS"
                      "TUVWXYZabc"
                      "defghijklm"
                      "nopqrstuvw"
                      "xyz";
    static int c = 1; /* Start with 'A' */
    char* s;
#if 0
    int len;
    int a;
    int b;
    char* p;
    if(c>=63*63*63){
        len = 4;
    }else if(c>=63*63){
        len = 3;
    }else if(c>=63){
        len = 2;
    }else{
        len = 1;
    }
    s = u_new(char,len+1);
    for(a=c;a>0;a/=63)
        *p++ = a%63;
#else
    if(c>=53*63*63){
        s = u_new(char,5);
        s[0] = ch[10+c/(53*63*63)];
        s[1] = ch[(c%53)/(63*63)];
        s[2] = ch[(c%(53*63))/63];
        s[3] = ch[c%(53*63*63)];
        s[4] = '\0';
    }else if(c>=53*63){
        s = u_new(char,4);
        s[0] = ch[10+c/(53*63)];
        s[1] = ch[(c%53)/63];
        s[2] = ch[c%(53*63)];
        s[3] = '\0';
    }else if(c>=53){
        s = u_new(char,3);
        s[0] = ch[10+c/53];
        s[1] = ch[c%53];
        s[2] = '\0';
    }else{
        s = u_new(char,2);
        s[0] = ch[10+c];
        s[1] = '\0';

    }
#endif
    ++c;
    return s;
}


static UpProperty* property_new(const char* name,const char* value){
    UpProperty* prop = u_new(UpProperty,1);
    prop->name = name;
    prop->value = u_strdup(value);
    return prop;
}


static void property_free(UpProperty* p){
    u_free(p->value);
    u_free(p);
}


static UpEvent* event_new(const char* name,const char* handler){
    UpEvent* e = u_new(UpEvent,1);
    e->name = name;
    e->handler = u_strdup(handler);
    return e;
}


static void event_free(UpEvent* e){
    u_free(e->handler);
    u_free(e);
}


static UpModel* model_new(UpModelType type){
    UpModel* m = u_new(UpModel,1);
    m->id = register_identifier();
    m->type = type;
    return m;
}


static UpWidget* widget_new(UpWidgetType type){
    UpWidget* w = u_new(UpWidget,1);
    w->id = register_identifier();
    w->type = type;
    w->property_list = NULL;
    w->event_list = NULL;
    return w;
}


static void widget_add_property(UpWidget* w,const char* name,const char* value){
    w->property_list = list_prepend(w->property_list,property_new(name,value));
}


static void widget_add_event(UpWidget* w,const char* name,const char* handler){
    w->event_list = list_prepend(w->event_list,event_new(name,handler));
}

/* --- model ------------------------------------------------------------ */

UpModel* up_model_new_int(int n){
    UpModel* m = model_new(UP_INT);
    return m;
}


UpModel* wgui_model_new_string(const char* s){
    UpModel* m = model_new(UP_STRING);
    return m;
}


UpModel* wgui_model_new_list(void){
    UpModel* m = model_new(UP_LIST);
    return m;
}

/* --- widget ----------------------------------------------------------- */

UpWidget* up_widget_new_label(const char* text){
    UpWidget* w = widget_new(UP_LABEL);
    widget_add_property(w,"text",text);
    return w;
}


UpWidget* wgui_widget_new_button(const char* text,const char* on_click){
    UpWidget* w = widget_new(UP_BUTTON);
    widget_add_property(w,"text",text);
    widget_add_event(w,"click",on_click);
    return w;
}


UpWidget* wgui_widget_new_button_box(){
    return widget_new(UP_BUTTON_BOX);
}


UpWidget* wgui_widget_new_vbox(){
    return widget_new(UP_VBOX);
}


UpWidget* wgui_widget_new_hbox(){
    return widget_new(UP_HBOX);
}


UpWidget* wgui_widget_new_listview(){
    return widget_new(UP_LISTVIEW);
}


void wgui_widget_free(UpWidget* w){
    u_free(w->id);
    list_free_full(w->property_list,(ListNotifyFunc)property_free);
    list_free_full(w->event_list,(ListNotifyFunc)event_free);
    u_free(w);
}


UpProperty* up_widget_get_property(UpWidget* w,const char* name){
    List* iter;
    for(iter=w->property_list;iter;iter=iter->next){
        UpProperty* p = (UpProperty*)iter->data;
        if(!strcmp(p->name,name))
            return p;
    }
    return NULL;
}


UpEvent* up_widget_get_event(UpWidget* w,const char* name){
    List* iter;
    for(iter=w->event_list;iter;iter=iter->next){
        UpEvent* e = (UpEvent*)iter->data;
        if(!strcmp(e->name,name))
            return e;
    }
    return NULL;
}
