/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <lauxlib.h>
#include <lua.h>
#include <lualib.h>

#include "debug.h"
#include "genjs.h"
#include "wgui.h"


static void regfunc(lua_State* lua,const char* name,lua_CFunction func){
    lua_pushcfunction(lua,func);
    lua_setglobal(lua,name);
}


#if 0
static void lua_pushpointer(lua_State* lua,void* p){
    void** bp = lua_newuserdata(lua,sizeof(p));
    *bp = p;
    lua_pushuserdata(lua,bp);
}
#endif


#if 0
static void* lua_topointer(lua_State* lua,int n){
    void** bp = luaL_checkuserdata(lua,n);
    return *bp;
}
#endif


static int add_label(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_label(luaL_checkstring(lua,3));
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int label_set(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    genjs_label_set(req,luaL_checkstring(lua,2),luaL_checkstring(lua,3));
    return 1;
}


static int add_button(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_button(luaL_checkstring(lua,3),luaL_checkstring(lua,4));
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int add_button_box(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_button_box();
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int add_vbox(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_vbox();
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int add_hbox(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_hbox();
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int add_listview(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    WguiWidget* w = wgui_widget_new_listview();
    lua_pushstring(lua,genjs_add_widget(req,luaL_checkstring(lua,2),w));
    wgui_widget_free(w);
    return 1;
}


static int listview_add(lua_State* lua){
    WguiRequest* req = (WguiRequest*)lua_topointer(lua,1);
    genjs_listview_add(req,luaL_checkstring(lua,2),luaL_checkstring(lua,3));
    return 1;
}

/* --- public C API ----------------------------------------------------- */

void wgui_lua_init(lua_State* lua){
    regfunc(lua,"add_label",add_label);
    regfunc(lua,"label_set",label_set);
    regfunc(lua,"add_button",add_button);
    regfunc(lua,"add_button_box",add_button_box);
    regfunc(lua,"add_vbox",add_vbox);
    regfunc(lua,"add_hbox",add_hbox);
    regfunc(lua,"add_listview",add_listview);
    regfunc(lua,"listview_add",listview_add);
}


void wgui_lua_done(){
}



void wgui_lua_call0(lua_State* lua,const char* func,WguiRequest* req){
    lua_getglobal(lua,func);
    lua_pushlightuserdata(lua,req);

    /* do the call (1 argument, 0 result) */
    if(lua_pcall(lua,1,0,0)){
        DEBUG_p("error running function %s: %s",func,lua_tostring(lua,-1));
    }
}


void wgui_lua_call1s(lua_State* lua,const char* func,WguiRequest* req,const char* s1){
    lua_getglobal(lua,func);
    lua_pushlightuserdata(lua,req);
    lua_pushstring(lua,s1);

    /* do the call (2 arguments, 0 result) */
    if(lua_pcall(lua,2,0,0)){
        DEBUG_p("error running function %s: %s",func,lua_tostring(lua,-1));
    }
}
