/*
 * Copyright (C) 2015 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "atomic.h"
#include "debug.h"
#include "hash.h"
#include "memutil.h"
#include "uref.h"


#ifdef DEBUG
typedef struct{
    int* refcount;
    char* string;
}DebugRef;


static HashMap* hash;


static void debug_ref_free(DebugRef* rd){
    u_free(rd->string);
    u_free(rd);
}


void DEBUG_ref_init(){
    hash = hash_map_new(direct_hash,direct_equal,NULL,(HashMapNotifyFunc)debug_ref_free);
}


static int DEBUG_ref_done_cb(void* key,void* value,void* userdata){
    DebugRef* dr = (DebugRef*)value;
    int* count = (int*)userdata;
    DEBUG_p("living object: <%p> [%d]: %s",key,*dr->refcount,dr->string);
    (*count)++;
    return 0;
}
void DEBUG_ref_done(){
    int count;

    count = 0;
#if 1
    hash_map_foreach(hash,(HashMapForeachFunc)DEBUG_ref_done_cb,&count);
#else
    while(g_hash_table_iter_next(&iter,&key,&value)){
        Ref* ref = (Ref*)value;
        DEBUG_p("living object: <%p> [%d]: %s",key,*ref->refcount,ref->string);
        ++count;
    }
#endif
    DEBUG_p("living objects: %d",count);
    hash_map_free(hash);
}


void DEBUG_ref_reg_full(void* obj,int* refcount,char* s){
    DebugRef* dr = u_new(DebugRef,1);
    dr->refcount = refcount;
    dr->string = s;
    hash_map_add(hash,obj,(void*)dr);
}


void DEBUG_ref_unreg(void* obj){
    hash_map_remove(hash,obj);
}
#endif /* DEBUG */


#if 0
URef* u_ref_init(URef* ref){
    ref->count = 1;
    DEBUG_ref_reg(ref,"Refable");
    return ref;
}
#endif


URef* u_ref_ref(URef* ref){
    if(ref->count>0)
        atomic_int_inc(&ref->count);
    return ref;
}


int u_ref_unref(URef* ref){
    int result;
    if(ref->count<=0)
        return 0;
    result = atomic_int_dec_and_test(&ref->count);
    if(result){
        DEBUG_ref_unreg(ref);
        ref->destroy(ref);
    }
    return result;
}
