/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../priv.h"

#include "../genjs.h"
#include "../memutil.h"
#include "../queue.h"
#include "box.h"
#include "expander.h"

typedef struct{
    UpWidget inh;
    Queue* child_queue;
    int spacing;
}UpBox;

/* --- genjs ------------------------------------------------------------ */

/* --- box --- */

void genjs_box_add(UpWidget* parent,UpWidget* child){
    Session* sess = parent->session;
    const char* child_name = genjs_get_element(child);
    if(child->expand_width)
        out("%s." EXPAND_WIDTH "=1;",child_name);
    if(child->expand_height)
        out("%s." EXPAND_HEIGHT "=1;",child_name);
    out("%s." CHILDREN ".push(%s);",genjs_get_element(parent),child_name);
}

/* --- hbox --- */

static void create_hbox_request_geometry(Session* sess,const char* hbox_request_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    out("%s=function(e){",hbox_request_geometry);
    out(  "e.w=0;"
          "e.h=0;"
          "e." EXPAND_WIDTH "=0;"
          "e." EXPAND_HEIGHT "=0;"
          "var a=e." CHILDREN ".length-1;"
          "if(a>0)"
            "e.w+=a*e." SPACING ";"
          "e." CHILDREN ".forEach(function(c){");
    out(    "%s(c);",element_request_geometry);
    out(    "e.w+=c.w;"
            "if(c.h>e.h)"
              "e.h=c.h;"
            "e." EXPAND_WIDTH "+=c." EXPAND_WIDTH ";"
            "if(c." EXPAND_WIDTH ">e." EXPAND_WIDTH ")"
              "e." EXPAND_WIDTH "=c." EXPAND_WIDTH ";"
          "});"
        "}");
}
#define get_hbox_request_geometry() genjs_get_function(sess,create_hbox_request_geometry)


static void create_hbox_set_geometry(Session* sess,const char* hbox_set_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    const char* element_set_geometry = genjs_get_element_set_geometry();
    out("%s=function(e,x,y,w,h){",hbox_set_geometry);
    out(  "var a=e." CHILDREN ".length-1;"
          "if(a>0)"
            "w-=a*e." SPACING ";"
          "var size=0;"
          "var exp=0;"
          "e." CHILDREN ".forEach(function(c){");
    out(    "%s(c);",element_request_geometry);
    out(    "size+=c.w;"
            "exp+=c." EXPAND_WIDTH ";"
          "});"
          "var ext=w-size;"
          "e." CHILDREN ".forEach(function(c){"
            "if(c!=e." CHILDREN "[0])"
              "x+=e." SPACING ";"
            "if(c." EXPAND_WIDTH "){"
              "w=(ext*c." EXPAND_WIDTH ")/exp|0;"
              "exp-=c." EXPAND_WIDTH ";"
              "ext-=w;"
            "}else{"
              "w=0;"
            "}"
            "w+=c.w;");
    out(    "%s(c,x,y,w,h);",element_set_geometry);
    out(    "x+=w;"
          "});"
        "}");
}
#define get_hbox_set_geometry() genjs_get_function(sess,create_hbox_set_geometry);


#if 0
static void create_hbox_new(Session* sess,const char* hbox_new){
    const char* hbox_request_geometry = get_hbox_request_geometry();
    const char* hbox_set_geometry = get_hbox_set_geometry();
    out("function %s(){",hbox_new);
    out(  "return {" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s," CHILDREN ":[]};",hbox_request_geometry,hbox_set_geometry);
    out("}");
}
#define get_hbox_new() genjs_get_function(sess,create_hbox_new)
#endif


void genjs_hbox_create(UpWidget* w,const char* name){
    Session* sess = w->session;
#if 1
    UpBox* b = (UpBox*)w;
    const char* hbox_request_geometry = get_hbox_request_geometry();
    const char* hbox_set_geometry = get_hbox_set_geometry();
    out("%s={" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s," CHILDREN ":[]," SPACING ":%d};",name,hbox_request_geometry,hbox_set_geometry,b->spacing);
    if(w->margin){
        out("%s.pw=%d",name,w->margin*2);
        out("%s.ph=%d",name,w->margin*2);
    }
#else
    out("%s=%s()",name,get_hbox_new());
#endif
}

/* --- vbox --- */

static void create_vbox_request_geometry(Session* sess,const char* vbox_request_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    out("%s=function(e){",vbox_request_geometry);
    out(  "e.w=0;"
          "e.h=0;"
          "e." EXPAND_WIDTH "=0;"
          "e." EXPAND_HEIGHT "=0;"
          "e." CHILDREN ".forEach(function(c){");
    out(    "%s(c);",element_request_geometry);
    out(    "if(c.w>e.w)"
              "e.w=c.w;"
            "e.h+=c.h;"
            "if(c." EXPAND_WIDTH ">e." EXPAND_WIDTH ")"
              "e." EXPAND_WIDTH "=c." EXPAND_WIDTH ";"
            "e." EXPAND_HEIGHT "+=c." EXPAND_HEIGHT ";"
          "});"
        "}");
}
#define get_vbox_request_geometry() genjs_get_function(sess,create_vbox_request_geometry)


static void create_vbox_set_geometry(Session* sess,const char* vbox_set_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    const char* element_set_geometry = genjs_get_element_set_geometry();
    out("%s=function(e,x,y,w,h){",vbox_set_geometry);
    out(  "var a=e." CHILDREN ".length-1;"
          "if(a>0)"
            "h-=a*e.sp;"
          "var size=0;"
          "var exp=0;"
          "e." CHILDREN ".forEach(function(c){");
    out(    "%s(c);",element_request_geometry);
    out(    "size+=c.h;"
            "exp+=c." EXPAND_HEIGHT ";"
          "});"
          "var ext=h-size;"
          "e." CHILDREN ".forEach(function(c){"
            "if(c!=e." CHILDREN "[0])"
              "y+=e." SPACING ";"
            "if(c." EXPAND_HEIGHT "){"
              "h=(ext*c." EXPAND_HEIGHT ")/exp|0;"
              "exp-=c." EXPAND_HEIGHT ";"
              "ext-=h;"
            "}else{"
              "h=0;"
            "}"
            "h+=c.h;");
    out(    "%s(c,x,y,w,h);",element_set_geometry);
    out(    "y+=h;"
          "});"
        "}");
}
#define get_vbox_set_geometry() genjs_get_function(sess,create_vbox_set_geometry);


#if 0
static void create_vbox_new(Session* sess,const char* vbox_new){
    const char* vbox_request_geometry = get_vbox_request_geometry();
    const char* vbox_set_geometry = get_vbox_set_geometry();
    out("function %s(){",vbox_new);
    out(  "return {" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s," CHILDREN ":[]};",vbox_request_geometry,vbox_set_geometry);
    out("}");
}
#define get_vbox_new() genjs_get_function(sess,create_vbox_new)
#endif


void genjs_vbox_create(UpWidget* w,const char* name){
    Session* sess = w->session;
#if 1
    UpBox* b = (UpBox*)w;
    const char* vbox_request_geometry = get_vbox_request_geometry();
    const char* vbox_set_geometry = get_vbox_set_geometry();
    out("%s={" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s," CHILDREN ":[]," SPACING ":%d};",name,vbox_request_geometry,vbox_set_geometry,b->spacing);
    if(w->margin){
        out("%s.pw=%d",name,w->margin*2);
        out("%s.ph=%d",name,w->margin*2);
    }
#else
    out("%s=%s()",name,get_vbox_new());
#endif
}

/* --- box widget ------------------------------------------------------- */


static void up_box_done(UpWidget* w){
    UpBox* b = (UpBox*)w;
    queue_free_full(b->child_queue,(QueueNotifyFunc)u_ref_unref);
    up_widget_done_func(w);
}


static void up_box_link(UpWidget* w){
    UpBox* b = (UpBox*)w;
    List* iter;
    for(iter=b->child_queue->head;iter;iter=iter->next){
        UpWidget* child = (UpWidget*)iter->data;
        child->session = w->session;
        genjs_box_add(w,child);
        up_widget_link(child);
    }
}


void up_box_add(UpWidget* parent,UpWidget* child){
    queue_append(((UpBox*)parent)->child_queue,child);
    child->parent = parent;
    if(parent->session){
        child->session = parent->session;
        genjs_box_add(parent,child);
        up_widget_link(child);
    }
}

void up_box_pack_start(UpWidget* parent,UpWidget* child,int expand,int fill,int a1){
    child->expand_width = expand;
    child->expand_height = expand;
    child->fill_width = fill;
    child->fill_height = fill;
    up_box_add(parent,child);
}


static UpWidget* up_box_new(int spacing){
    UpBox* b = u_new(UpBox,1);
    UpWidget* w = up_widget_init((UpWidget*)b);
    b->child_queue = queue_new();
    b->spacing = 0;
    return w;
}


UpWidget* up_hbox_new(int homo,int spacing){
    static const UpWidgetInterface up_hbox_vt = {
        .done = up_box_done,
        .genjs = genjs_hbox_create,
        .link = up_box_link
    };
    UpWidget* w = up_box_new(spacing);
    DEBUG_ref_reg((URef*)w,"Hbox");
    w->vt = &up_hbox_vt;
    return w;
}


UpWidget* up_vbox_new(int homo,int spacing){
    static const UpWidgetInterface up_vbox_vt = {
        .done = up_box_done,
        .genjs = genjs_vbox_create,
        .link = up_box_link
    };
    UpWidget* w = up_box_new(spacing);
    DEBUG_ref_reg((URef*)w,"Vbox");
    w->vt = &up_vbox_vt;
    return w;
}


UpWidget* up_button_box_new(){
    static const UpWidgetInterface up_button_box_vt = {
        .done = up_box_done,
        .genjs = genjs_hbox_create,
        .link = up_box_link
    };
    UpWidget* w = up_box_new(6);
    DEBUG_ref_reg((URef*)w,"ButtonBox");
    w->vt = &up_button_box_vt;
    w->margin = 6;
    up_box_add(w,up_expander_new(1,0));
    return w;
}
