/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../genjs.h"
#include "../memutil.h"
#include "button.h"
#include "label.h"

typedef struct{
    UpContainer inh;
    //char* label;
    Signal signal_activate;
}UpButton;

/* --- genjs ------------------------------------------------------------ */

void genjs_button_create_container(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    //UpButton* button = (UpButton*)w;
    out("%s=document.createElement('a');",name);
    out("%s.className='button'",name);
    out("%s(%s);",element_init,name);
    genjs_container_do_child(w);
}


static void button_on_activate(void* userdata,Session* sess,const char* args){
    UpButton* b = (UpButton*)userdata;
    signal_invoke(&b->signal_activate,userdata);
}

void genjs_button_connect_activate(UpWidget* w){
    Session* sess = w->session;
    const char* name = genjs_get_element(w);
    out("%s.onclick=function(){_('%s')};",name,session_register_js_callback(sess,button_on_activate,w));
}


void genjs_button_disconnect_activate(UpWidget* w){
    Session* sess = w->session;
    out("%s.removeAttribute('onclick');",genjs_get_element(w));
}

/* --- button widget ---------------------------------------------------- */

/*
void up_button_on_activate(void* userdata,Session* sess,const char* args){
    UpButton* b = (UpButton*)userdata;
    signal_invoke(&b->signal_activate,NULL);
}
*/


static void up_button_done_func(UpWidget* w){
    UpButton* b = (UpButton*)w;
    //u_free_if_set(b->label);
    signal_done(&b->signal_activate);
}


static void up_button_link(UpWidget* w){
    UpButton* b = (UpButton*)w;
    up_container_link_func(w);
    if(b->signal_activate.subscription_list)
        genjs_button_connect_activate(w);
}


UpWidget* up_button_new(){
    static const UpWidgetInterface up_button_vt = {
        .done = up_button_done_func,
        .genjs = genjs_button_create_container,
        .link = up_button_link
    };
    UpButton* b = u_new(UpButton,1);
    UpWidget* w = up_container_init_func((UpWidget*)b);
    DEBUG_ref_reg((URef*)w,"Button");
    w->vt = &up_button_vt;
    signal_init(&b->signal_activate);
    return w;
}


UpWidget* up_button_new_x(const char* text,SignalHandler func){
    UpButton* b = (UpButton*)up_button_new();
    if(text){
        UpWidget* l = up_label_new(text);
        up_container_add((UpWidget*)b,l);
    }
    if(func){
        signal_subscribe(&b->signal_activate,func,NULL);
        //genjs_button_connect_activate((UpWidget*)b);
    }
    return (UpWidget*)b;
}


SignalSubscription* up_button_subscribe_activate(UpWidget* w,SignalHandler handler,void* userdata){
    UpButton* b = (UpButton*)w;
    if(w->session && !b->signal_activate.subscription_list)
        genjs_button_connect_activate(w);
    return signal_subscribe(&b->signal_activate,handler,userdata);
}


void up_button_unsubscribe_activate(UpWidget* w,SignalSubscription* ss){
    UpButton* b = (UpButton*)w;
    signal_unsubscribe(&b->signal_activate,ss);
    if(w->session && !b->signal_activate.subscription_list)
        genjs_button_disconnect_activate(w);
}
