/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../genjs.h"
#include "../memutil.h"
#include "container.h"

/* --- genjs -------------------------------------------------------- */

void genjs_create_container_request_geometry(Session* sess,const char* container_request_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    out("%s=function(e){",container_request_geometry);
    out(  "%s(e." CHILD ");",element_request_geometry);
    out(  "e.w=e." CHILD ".w;"
          "e.h=e." CHILD ".h;");
    out(  "e." EXPAND_WIDTH "=e." CHILD "." EXPAND_WIDTH ";"
          "e." EXPAND_HEIGHT "=e." CHILD "." EXPAND_HEIGHT ";");
    out("}");
}


void genjs_create_container_set_geometry(Session* sess,const char* container_set_geometry){
    const char* element_set_geometry = genjs_get_element_set_geometry();
    out("%s=function(e,x,y,w,h){",container_set_geometry);
    out(  "%s(e." CHILD ",x,y,w,h);",element_set_geometry);
    out("}");
}
#define get_container_set_geometry() genjs_get_function(sess,create_container_set_geometry)


void genjs_container_do_child(UpWidget* w){
    UpContainer* c = (UpContainer*)w;
    if(c->child){
        c->child->session = w->session;
        genjs_container_add(w,c->child);
    }
}


void genjs_container_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    //UpContainer* c = (UpContainer*)w;
    const char* container_request_geometry = genjs_get_container_request_geometry();
    const char* container_set_geometry = genjs_get_container_set_geometry();
    out("%s={" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s};",name,container_request_geometry,container_set_geometry);
    if(w->margin){
        out("%s.pw=%d",name,w->margin*2);
        out("%s.ph=%d",name,w->margin*2);
    }
}


void genjs_container_add(UpWidget* parent,UpWidget* child){
    Session* sess = parent->session;
    const char* container_request_geometry = genjs_get_container_request_geometry();
    const char* container_set_geometry = genjs_get_container_set_geometry();
    const char* parent_name = genjs_get_element(parent);
    const char* child_name = genjs_get_element(child);
    if(child->expand_width)
        out("%s." EXPAND_WIDTH "=1;",child_name);
    if(child->expand_height)
        out("%s." EXPAND_HEIGHT "=1;",child_name);
    if(child->fill_width)
        out("%s." FILL_WIDTH "=1;",child_name);
    if(child->fill_height)
        out("%s." FILL_HEIGHT "=1;",child_name);
    out("%s." CHILD "=%s;",parent_name,child_name);
    out("%s." GEOMETRY_REQ "=%s;",parent_name,container_request_geometry);
    out("%s." GEOMETRY_SET "=%s;",parent_name,container_set_geometry);
}

/* --- container widget --------------------------------------------- */

UpWidget* up_container_init_func(UpWidget* w){
    UpContainer* c = (UpContainer*)up_widget_init(w);
    c->child = NULL;
    return w;
}


void up_container_link_func(UpWidget* w){
    UpContainer* c = (UpContainer*)w;
    if(c->child){
        c->child->session = w->session;
        c->child->expand_width = w->expand_width;
        c->child->expand_height = w->expand_height;
        genjs_container_add(w,c->child);
        up_widget_link(c->child);
    }
}


UpWidget* up_container_new(){
    static const UpWidgetInterface up_container_vt = {
        .done = up_widget_done_func,
        .genjs = genjs_container_create,
        .link = up_container_link_func
        //.request_geometry = up_container_request_geometry_func,
        //.set_geometry = up_container_set_geometry_func
    };
    UpWidget* w = up_container_init_func((UpWidget*)u_new(UpContainer,1));
    DEBUG_ref_reg((URef*)w,"Container");
    w->vt = &up_container_vt;
    return w;
}


void up_container_add(UpWidget* parent,UpWidget* child){
    ((UpContainer*)parent)->child = child;
    child->parent = parent;
    if(parent->session)
        up_container_link_func(parent);
}
