/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../debug.h"
#include "../fileinfo.h"
#include "../upstock.h"
#include "../genjs.h"
#include "../memutil.h"
#include "image.h"

typedef struct{
    UpWidget inh;
    int width;
    int height;
    char* mimetype;
    char* filename;
}UpImage;

/* --- genjs -------------------------------------------------------- */

void genjs_image_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    UpImage* p = (UpImage*)w;
    out("%s=document.createElement('img');",name);
    out("%s.width=%s.mw='%d';",name,name,p->width);
    out("%s.height=%s.mh='%d';",name,name,p->height);
    out("%s.src='%s';",name,session_register_file_callback(sess,up_image_on_file_request,w));
    out("%s.style.pointerEvents='none';",name);
    out("%s(%s);",element_init,name);
}

/* --- image widget ------------------------------------------------- */

#if 1
void up_image_on_file_request(void* userdata,char** filename,char** mimetype){
    UpImage* p = (UpImage*)userdata;
    DEBUG_p("sending file: %s",p->filename);
    *filename = p->filename?u_strdup(p->filename):NULL;
    *mimetype = p->mimetype?u_strdup(p->mimetype):NULL;
}
#else
void up_image_on_data_request(void* userdata,UpSession* sess,const char* args){
    UpImage* p = (UpImage*)userdata;
    DEBUG_p("sending file: %s",p->filename);
    sess->data.filename = p->filename?u_strdup(p->filename):NULL;
    sess->data.mimetype = p->mimetype?u_strdup(p->mimetype):NULL;
}
#endif


static void up_image_done(UpWidget* w){
    UpImage* p = (UpImage*)w;
    u_free_if_set(p->filename);
    u_free_if_set(p->filename);
}


UpWidget* up_image_new(const char* filename){
    static const UpWidgetInterface up_image_vt = {
        .done = up_image_done,
        .genjs = genjs_image_create
    };
    UpImage* p = u_new(UpImage,1);
    UpWidget* w = up_widget_init((UpWidget*)p);
    DEBUG_ref_reg((URef*)w,"Image");
    w->vt = &up_image_vt;
    w->fill_width = 0;
    w->fill_height = 0;
    get_imageinfo(filename,&p->width,&p->height,&p->mimetype);
    p->filename = u_strdup(filename);
    return w;
}


UpWidget* up_image_new_from_stock(const char* id,int size){
    static const UpWidgetInterface up_image_vt = {
        .done = up_image_done,
        .genjs = genjs_image_create
    };
    UpImage* p = u_new(UpImage,1);
    UpWidget* w = up_widget_init((UpWidget*)p);
    DEBUG_ref_reg((URef*)w,"Image");
    w->vt = &up_image_vt;
    w->fill_width = 0;
    w->fill_height = 0;
    p->filename = up_stock_get_filename(id,size);
    if(p->filename){
        p->width = size;
        p->height = size;
        get_imageinfo(p->filename,NULL,NULL,&p->mimetype);
    }
    return w;
}
