/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../genjs.h"
#include "../memutil.h"
#include "label.h"

typedef struct{
    UpWidget inh;
    char* text;
}UpLabel;

/* --- genjs -------------------------------------------------------- */

void genjs_label_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    UpLabel* l = (UpLabel*)w;
    out("%s=document.createElement('span');",name);
    if(l->text)
        out("%s.appendChild(document.createTextNode('%s'))",name,l->text);
    out("%s.style.pointerEvents='none';",name);
    out("%s(%s);",element_init,name);
}


void genjs_label_set(UpWidget* w){
    Session* sess = w->session;
    const char* name = genjs_get_element(w);
    UpLabel* l = (UpLabel*)w;
    out("while(%s.firstChild)"
          "%s.removeChild(%s.firstChild);",name,name,name);
    if(l->text)
        out("%s.appendChild(document.createTextNode('%s'))",name,l->text);
}

/* --- label widget ------------------------------------------------- */

static void up_label_done_func(UpWidget* w){
    UpLabel* l = (UpLabel*)w;
    u_free_if_set(l->text);
}


UpWidget* up_label_new(const char* text){
    static const UpWidgetInterface up_label_vt = {
        .done = up_label_done_func,
        .genjs = genjs_label_create,
    };
    UpLabel* l = u_new(UpLabel,1);
    UpWidget* w = up_widget_init((UpWidget*)l);
    DEBUG_ref_reg((URef*)w,"Label");
    w->vt = &up_label_vt;
    l->text = text?u_strdup(text):NULL;
    return w;
}
