/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../priv.h"

#include "../genjs.h"
#include "../memutil.h"
#include "listview.h"

typedef struct{
    UpWidget inh;
    ListModel* model;
    Signal signal_select;
}UpListView;

/* --- genjs ------------------------------------------------------------ */

static void create_list_view_select_item(Session*,const char*);
#define get_list_view_select_item() genjs_get_function(sess,create_list_view_select_item)
static void create_list_view_select(Session*,const char*);
#define get_list_view_select() genjs_get_function(sess,create_list_view_select)


static void create_list_view_add(Session* sess,const char* list_view_add){
    const char* list_view_select_item = get_list_view_select_item();
    out("%s=function(w,i,v){",list_view_add);
    out(  "var a=document.createElement('li');"
          "a.appendChild(document.createTextNode(v));"
          "a.onclick=function(){%s(w,a);};",list_view_select_item); /* FIXME!! can get the wrong index */
    out(  "if(i<0||i>=w.children.length){"
            "w.appendChild(a)"
          "}else{"
            "w.insertBefore(item.w.children[index]);"
          "}"
        "}");
}
#define get_list_view_add(a) genjs_get_function(sess,create_list_view_add)


static void create_list_view_del(Session* sess,const char* name){
   out("%s=function(w,i){",name);
   out(  "if(i<0||i>=w.children.length)"
           "return;"
         "w.removeChild(w.children[i]);"
       "}");
}
#define get_list_view_del(a) genjs_get_function(sess,create_list_view_del)


static void create_list_view_select_item(Session* sess,const char* name){
    const char* list_view_select = get_list_view_select();
    out("%s=function(w,a){",name);
    out(  "for(var i=0;i<w.children.length;++i)"
            "if(w.children[i]===a){"
              "%s(w,i);",list_view_select);
    out(      "return;"
            "}"
        "}");
}


static void create_list_view_select(Session* sess,const char* list_view_select){
    out("%s=function(w,i){",list_view_select);
    out(  "if('selected' in w)"
            "w.children[w.selected].className='';"
          "w.children[i].className='selected';"
          "w.selected=i;"
        "}");
}


static void create_list_view_link_model(Session* sess,const char* name){
    const char* list_view_add = get_list_view_add();
    const char* list_view_del = get_list_view_del();
    out("%s=function(w,m){",name);
    out(  "for(var i=0;i<m.item.length;++i)"
            "%s(w,-1,m.item[i]);",list_view_add);
    out(  "m.on_add.push(function(i,v){%s(w,i,v);});",list_view_add);
    out(  "m.on_del.push(function(i){%s(w,i);});",list_view_del);
    out("}");
}
#define get_list_view_link_model() genjs_get_function(sess,create_list_view_link_model)


void genjs_list_view_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    UpListView* view = (UpListView*)w;
    out("%s=document.createElement('ul');",name);
    out("%s.className='listview';",name);
    out("%s(%s);",element_init,name);
    if(view->model){
        const char* func1 = get_list_view_link_model();
        const char* name1 = genjs_get_list_model(sess,view->model);
        out("%s(%s,%s);",func1,name,name1);
    }
}

/* --- list view widget ------------------------------------------------- */

static void up_list_view_done_func(UpWidget* w){
    list_model_unref(((UpListView*)w)->model);
}


UpWidget* up_list_view_new(ListModel* m){
    static const UpWidgetInterface up_list_view_vt = {
        .done = up_list_view_done_func,
        .genjs = genjs_list_view_create,
    };
    UpListView* lv = u_new(UpListView,1);
    //UpView* v = up_view_init_func(C_VIEW(
    UpWidget* w = up_widget_init((UpWidget*)lv);
    DEBUG_ref_reg((URef*)w,"ListView");
    w->vt = &up_list_view_vt;
    lv->model = m;
    signal_init(&lv->signal_select);
    return w;
}
