/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../priv.h"

#include "../debug.h"
#include "../genjs.h"
#include "../memutil.h"
#include "root.h"

typedef struct{
    UpContainer inh;
    Signal signal_key;
}UpRoot;

/* --- genjs ------------------------------------------------------------ */

typedef struct{
    int key;
}KeyData;
static int root_on_key_marshal(SignalHandler handler,void* userdata,void* signaldata){
    KeyData* d = (KeyData*)signaldata;
    return ((UpKeyHandler)handler)(d->key,userdata);
}
static void root_on_key(void* userdata,Session* sess,const char* args){
    UpRoot* r = (UpRoot*)userdata;
    KeyData d;
    DEBUG_p("args: '%s'",args);
    d.key = atoi(args);
    signal_invoke_marshal(&r->signal_key,root_on_key_marshal,&d);
}


void genjs_root_connect_key(UpWidget* w){
    Session* sess = w->session;
    out("document.onkeypress=function(k){_('%s?'+k.charCode)};",session_register_js_callback(sess,root_on_key,w));
}


void genjs_root_disconnect_key(UpWidget* w){
    Session* sess = w->session;
    out("document.removeAttribute('onkeypress');");
}

/* --- root widget ------------------------------------------------------ */

UpWidget* up_root_new(Session* sess){
    static const UpWidgetInterface up_root_vt = {
        .done = up_widget_done_func,
        .genjs = NULL, /* pre-existing as the body element */
        .link = up_container_link_func
        //.request_geometry = up_container_request_geometry_func,
        //.set_geometry = up_container_set_geometry_func
    };
    UpRoot* r = u_new(UpRoot,1);
    UpWidget* w = up_container_init_func((UpWidget*)r);
    DEBUG_ref_reg((URef*)w,"Root");
    w->vt = &up_root_vt;
    w->session = sess;
    signal_init(&r->signal_key);
    hash_map_add(sess->jsid_map,w,u_strdup("document.body"));
    return w;
}


SignalSubscription* up_root_subscribe_key(UpWidget* w,UpKeyHandler handler,void* userdata){
    UpRoot* r = (UpRoot*)w;
    if(!r->signal_key.subscription_list)
        genjs_root_connect_key(w);
    return signal_subscribe(&r->signal_key,(SignalHandler)handler,userdata);
}


void up_root_unsubscribe_key(UpWidget* w,SignalSubscription* ss){
    UpRoot* r = (UpRoot*)w;
    signal_unsubscribe(&r->signal_key,ss);
    if(!r->signal_key.subscription_list)
        genjs_root_disconnect_key(w);
}
