/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../priv.h"

#include "../genjs.h"
#include "../memutil.h"
#include "scale.h"

typedef struct{
    UpWidget inh;
    RangeModel* model;
}UpScale;

/* --- genjs ------------------------------------------------------------ */

static void genjs_create_hscale_set_geometry(Session* sess,const char* hscale_set_geometry){
    const char* element_set_geometry = genjs_get_element_set_geometry();
    out("%s=function(s,x,y,w,h){",hscale_set_geometry);
    out(  "s.s.style.left=x+'px';"
          "s.s.style.top=y+'px';"
          "s.x1=x;"
          "s.x2=x+w-16-s.s.pw;"
          "if(s.m)"
            "x+=(s.x2-s.x1)*(s.m.v-s.m.min)/(s.m.max-s.m.min)|0;");
    out(  "%s(s.s,x,y,16,16);",element_set_geometry);
    out("}");
}
#define genjs_get_hscale_set_geometry() genjs_get_function(sess,genjs_create_hscale_set_geometry)


static void genjs_create_hscale_update(Session* sess,const char* hscale_update){
    out("%s=function(s){",hscale_update);
    out(  "var x=parseInt(s.style.left);"
          "if(s.m)"
            "x+=(s.x2-s.x1)*(s.m.v-s.m.min)/(s.m.max-s.m.min)|0;"
          "s.s.style.left=x+'px';"
        "}");
}
#define genjs_get_hscale_update() genjs_get_function(sess,genjs_create_hscale_update)


static void genjs_create_hscale_link_model(Session* sess,const char* hscale_link_model){
    const char* hscale_update = genjs_get_hscale_update();
    out("%s=function(s,m){",hscale_link_model);
    out(  "s.m=m;");
    out(  "m.ons.push(function(){%s(s)});",hscale_update);
    out(  "m.onl.push(function(){%s(s)});",hscale_update);
    out("}");
}
#define genjs_get_hscale_link_model() genjs_get_function(sess,genjs_create_hscale_link_model)


static void genjs_create_hscale_begin_move(Session* sess,const char* hscale_begin_move){
    const char* range_model_set = genjs_get_range_model_set();
    out("%s=function(s,e){",hscale_begin_move);
    out(  "e=e||window.event;"
          "var x=parseInt(s.s.style.left);"
          "var x1=e.clientX;"
          "document.onmousemove=function(e){"
            "e=e||window.event;"
            "var x2=e.clientX;"
            "x+=x2-x1;"
            "x1=x2;"
            "if(x<s.x1)"
              "x=s.x1;"
            "if(x>s.x2)"
              "x=s.x2;"
            "if(s.m){");
    out(      "%s(s.m,(s.m.max-s.m.min)*(x-s.x1)/(s.x2-s.x1)|0);",range_model_set);
    out(    "}else{"
              "s.s.style.left=x+'px';"
            "}"
          "};"
          "document.onmouseup=function(){"
            "document.onmousemove=null;"
            "document.onmouseup=null;"
          "}"
        "}");
}
#define genjs_get_hscale_begin_move() genjs_get_function(sess,genjs_create_hscale_begin_move)


void genjs_hscale_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    const char* hscale_begin_move = genjs_get_hscale_begin_move();
    const char* hscale_set_geometry = genjs_get_hscale_set_geometry();
    UpScale* sc = (UpScale*)w;
    out("var %s=document.createElement('div');",name);
    out("%s.className='scale';",name);
    out("%s(%s);",element_init,name);
    out("%s.s=document.createElement('div');",name);
    out("%s.s.className='slider';",name);
    out("%s(%s.s);",element_init,name);
    out("%s.s.mw+=16;",name);
    out("%s.s.mh+=16;",name);
    out("%s.mh=%s.s.mh+%s.s.ph+%s.ph;",name,name,name,name);
    out("%s.ew=1;",name);
    out("%s.s.onmousedown=function(e){%s(%s,e)};",name,hscale_begin_move,name);
    if(sc->model){
        const char* scale_link_model = genjs_get_hscale_link_model();
        const char* model_name = genjs_get_range_model(sess,sc->model);
        out("%s(%s,%s);",scale_link_model,name,model_name);
    }
    out("%s.ons=%s;",name,hscale_set_geometry);
}

/* --- scale widget ----------------------------------------------------- */

static void up_scale_done(UpWidget* w){
    range_model_unref(((UpScale*)w)->model);
    //up_widget_done_func(w);
}


UpWidget* up_hscale_new(RangeModel* m){
    static const UpWidgetInterface up_hscale_vt = {
        .done = up_scale_done,
        .genjs = genjs_hscale_create
    };
    UpScale* sc = u_new(UpScale,1);
    UpWidget* w = up_widget_init((UpWidget*)sc);
    DEBUG_ref_reg((URef*)w,"Hscale");
    w->vt = &up_hscale_vt;
    sc->model = m;
    return w;
}


/*
UpWidget* up_vscale_new(RangeModel* m){
}
*/
