/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../genjs.h"
#include "../memutil.h"
#include "speaker.h"

typedef struct{
    UpWidget inh;
    RangeModel* position;
    RangeModel* volume;
}UpSpeaker;

/* --- genjs ------------------------------------------------------------ */

void genjs_speaker_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    const char* element_init = genjs_get_element_init();
    UpSpeaker* p = (UpSpeaker*)w;
    out("%s=document.createElement('audio')",name);
    out("%s(%s);",element_init,name);
    if(p->position){
        const char* model_name = genjs_get_range_model(sess,p->position);
        const char* range_model_set = genjs_get_range_model_set();
        out("%s.p=%s",name,model_name);
        out("%s.ons.push(function(){%s.currentTime=%s.v});",model_name,name,model_name);
        out("%s.timeupdate=function(){%s(%s,%s.currentTime)};",name,range_model_set,model_name,name);
    }
    if(p->volume){
        const char* model_name = genjs_get_range_model(sess,p->volume);
        const char* range_model_set = genjs_get_range_model_set();
        out("%s.v=%s",name,model_name);
        out("%s.ons.push(function(){%s.volume=%s.v/100});",model_name,name,model_name);
        out("%s.volumechange=function(){%s(%s,%s.volume*100|0)};",name,range_model_set,model_name,name);
    }
}


void genjs_speaker_set_url(UpWidget* w,const char* url,const char* mimetype){
    Session* sess = w->session;
    const char* name = genjs_get_element(w);
    out("var a=document.createElement('source');");
    out("a.src='%s';",url);
    if(mimetype)
        out("a.type='%s';",mimetype);
    out("%s.appendChild(a);",name);
    out("%s.load();",name);
}


void genjs_speaker_play(UpWidget* w){
    Session* sess = w->session;
    out("%s.play();",genjs_get_element(w));
}


void genjs_speaker_pause(UpWidget* w){
    Session* sess = w->session;
    out("%s.pause();",genjs_get_element(w));
}

/* --- speaker widget --------------------------------------------------- */

/*
static void up_speaker_done_func(UpWidget* w){
}
*/


UpWidget* up_speaker_new(RangeModel* position,RangeModel* volume){
    static const UpWidgetInterface up_speaker_vt = {
        .done = up_widget_done_func,
        .genjs = genjs_speaker_create
    };
    UpSpeaker* p = u_new(UpSpeaker,1);
    UpWidget* w = up_widget_init((UpWidget*)p);
    DEBUG_ref_reg((URef*)w,"Speaker");
    w->vt = &up_speaker_vt;
    return w;
}


void up_speaker_set_url(UpWidget* w,const char* url,const char* mimetype){
    genjs_speaker_set_url(w,url,mimetype);
}


void up_speaker_play(UpWidget* w){
    genjs_speaker_play(w);
}


void up_speaker_pause(UpWidget* w){
    genjs_speaker_pause(w);
}
