/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../priv.h"

#include "../genjs.h"
#include "../memutil.h"
#include "../upmain.h"
#include "table.h"

typedef struct{
    UpWidget inh;
    int cols;
    int rows;
    Queue* child_queue;
    int spacing;
}UpTable;

/* --- genjs ------------------------------------------------------------ */

static void create_table_get_geometry(Session* sess,const char* table_get_geometry){
    const char* element_request_geometry = genjs_get_element_request_geometry();
    out("%s=function(e){",table_get_geometry);
    out(  "e.cols.forEach(function(col){"
            "col.w=0;"
            "col." EXPAND_WIDTH "=0;"
          "});"
          "e.rows.forEach(function(row){"
            "row.h=0;"
            "row." EXPAND_HEIGHT "=0;"
          "});");
          /* Init + Children spanning one cell */
    out(  "e." CHILDREN ".forEach(function(c){");
    out(    "%s(c);",element_request_geometry);
    out(    "if(c.cw<=1){"
              "var a=e.cols[c.cx];"
              "if(c.w>a.w)"
                "a.w=c.w;"
              "if(e." EXPAND_WIDTH ">a." EXPAND_WIDTH ")"
                "a." EXPAND_WIDTH "=c." EXPAND_WIDTH ";"
            "}"
            "if(c.ch<=1){"
              "var a=e.rows[c.cy];"
              "if(c.h>a.h)"
                "a.h=c.h;"
              "if(c." EXPAND_HEIGHT ">a." EXPAND_HEIGHT ")"
                "a." EXPAND_HEIGHT "=c." EXPAND_HEIGHT ";"
            "}"
          "});");
          /* Children spanning multiple cells */
    out(  "e." CHILDREN ".forEach(function(c){");
            /* Child spans multiple columns. */
    out(    "if(c.cw>1){");
            /* Check to see if there is enough space already. */
    out(      "var w=0;"
              "for(var i=0;i<c.cw;++i)"
                "w+=e.cols[c.cx+i].w;"
              "if(w<c.w){"
                "w=c.w-w;"
                "var exp=0;"
                "var force=0;"
                "for(var i=0;i<c.cw;++i)"
                  "if(e.cols[c.cx+i]." EXPAND_WIDTH ")"
                    "++exp;"
                "if(exp<=0){"
                  "exp=c.cw;"
                  "force=1;"
                "}"
                "for(var i=0;i<c.cw;++i)"
                  "if(force||e.cols[c.cx+i].ew){"
                    "var ext=w/exp;"
                    "e.cols[c.cx+i].w+=ext;"
                    "w-=ext;"
                    "--ext;"
                  "}"
              "}"
            "}");
            /* Child spans multiple rows. */
    out(    "if(c.ch>1){"
              "var h=0;"
              "for(var i=0;i<c.ch;++i)"
                "h+=e.rows[c.cy+i].h;");
              /* If we need to request more space for this child to fill... */
    out(      "if(h<c.h){"
                "h+=c.h-h;"
                "var exp=0;"
                "var force=0;"
                "for(var i=0;i<c.ch;++i)"
                  "if(e.rows[c.cy+i].eh)"
                    "++exp;"
                "if(exp<=0){"
                  "exp=c.ch;"
                  "force=1;"
                "}"
                "for(var i=0;i<c.ch;++i)"
                  "if(force||e.rows[c.cy+i].eh){"
                    "var ext=h/exp;"
                    "e.rows[c.cy+i].h+=ext;"
                    "h-=ext;"
                    "--ext;"
                  "}"
              "}"
            "}"
          "});"
        "}");
}
#define get_table_get_geometry() genjs_get_function(sess,create_table_get_geometry)


static void create_table_request_geometry(Session* sess,const char* table_request_geometry){
    const char* table_get_geometry = get_table_get_geometry();
    out("%s=function(e){",table_request_geometry);
    out(  "%s(e);",table_get_geometry);
    out(  "e.w=0;"
          "e." EXPAND_WIDTH "=0;"
          "var a=e.cols.length-1;"
          "if(a>0)"
            "e.w+=a*e.sp;"
          "e.cols.forEach(function(col){"
            "e.w+=col.w;"
            "e." EXPAND_WIDTH "+=col." EXPAND_WIDTH ";"
          "});"
          "e.h=0;"
          "e." EXPAND_HEIGHT "=0;"
          "a=e.rows.length-1;"
          "if(a>0)"
            "e.h+=a*e.sp;"
          "e.rows.forEach(function(row){"
            "e.h+=row.h;"
            "e." EXPAND_HEIGHT "+=row." EXPAND_HEIGHT ";"
          "});"
        "}");
}
#define get_table_request_geometry() genjs_get_function(sess,create_table_request_geometry)


static void create_table_set_geometry(Session* sess,const char* table_set_geometry){
    const char* table_get_geometry = get_table_get_geometry();
    const char* element_set_geometry = genjs_get_element_set_geometry();
    out("%s=function(e,x,y,w,h){",table_set_geometry);
    /* init */
    out(  "%s(e);",table_get_geometry);
    out(  "e.cols.forEach(function(col){"
            "col." EXPAND_WIDTH "=0;"
            "col.empty=1;"
            "col.n_exp=0;"
          "});"
          "e.rows.forEach(function(row){"
            "row." EXPAND_HEIGHT "=0;"
            "row.empty=1;"
            "row.n_exp=0;"
          "});");
          /* Children spanning one cell */
    out(  "e." CHILDREN ".forEach(function(c){"
            "if(c.cw<=1){"
              "if(c.ew)"
                "e.cols[c.cx]." EXPAND_WIDTH "=1;"
              "e.cols[c.cx].empty=0;"
            "}"
            "if(c.ch<=1){"
              "if(c.eh)"
                "e.rows[c.cy]." EXPAND_HEIGHT "=1;"
              "e.rows[c.cy].empty=0;"
            "}"
          "});");
          /* Children spanning multiple cells */
    out(  "e." CHILDREN ".forEach(function(c){"
            "if(c.cw>1){"
              "for(var i=0;i<c.cw;++i)"
                "e.cols[c.cx+i].empty=0;"
              "if(c.ew){"
                "var exp=0;"
                "for(var i=0;i<c.cw;++i)"
                  "if(e.cols[c.cx+i].ew){"
                    "exp = 1;"
                    "break;"
                  "}"
                "if(!exp)"
                  "for(var i=0;i<c.cw;++i)"
                    "e.cols[c.cx+i].n_exp=1;"
              "}"
            "}"
            "if(c.ch>1){"
              "for(var i=0;i<c.ch;++i)"
                "e.rows[c.cy+i].empty=0;"
              "if(c.eh){"
                "var exp=0;"
                "for(var i=0;i<c.ch;++i)"
                  "if(e.rows[c.cy+i].eh){"
                    "exp=1;"
                    "break;"
                  "}"
                "if(!exp)"
                  "for(var i=0;i<c.ch;++i)"
                    "e.rows[c.cy+i].n_exp=1;"
              "}"
            "}"
          "});");
    out(  "e.cols.forEach(function(col){"
            "if(col.empty){"
              "col." EXPAND_WIDTH "=0;"
            "}else if(col.n_exp){"
              "col." EXPAND_WIDTH "=1;"
            "}"
          "});"
          "e.rows.forEach(function(row){"
            "if(row.empty){"
              "col." EXPAND_HEIGHT "=0;"
            "}else if(row.n_exp){"
              "row." EXPAND_HEIGHT "=1;"
            "}"
          "});");
    /* calculate size of children */
    out(  "var size=0;"
          "var exp=0;"
          "e.cols.forEach(function(col){"
            "size+=col.w;"
            "exp+=col." EXPAND_WIDTH ";"
          "});"
          "if(w<size&&exp){"
            "var ext=w-size;"
            "e.cols.forEach(function(col){"
              "if(col." EXPAND_WIDTH "){"
                "size=(ext*col." EXPAND_WIDTH ")/exp;"
                "col.w+=size"
                "exp-=col." EXPAND_WIDTH ";"
                "ext-=size"
              "}"
            "});"
          "}"
          "size=0;"
          "exp=0;"
          "e.rows.forEach(function(row){"
            "size+=row.h;"
            "exp+=row." EXPAND_HEIGHT ";"
          "});"
          "if(h<size&&exp){"
            "var ext=h-size;"
            "e.rows.forEach(function(row){"
              "if(row." EXPAND_HEIGHT "){"
                "size=(ext*row." EXPAND_HEIGHT ")/exp;"
                "row.h+=size;"
                "exp-=row." EXPAND_HEIGHT ";"
                "ext-=size"
              "}"
            "});"
          "}");
    /* calculate position of children */
    out(  "e." CHILDREN ".forEach(function(c){"
            "var cx=x;"
            "for(var i=0;i<c.cx;++i)"
              "cx+=e.cols[i].w;"
            "var cw=0;"
            "for(var i=0;i<c.cw;++i)"
              "cw+=e.cols[c.cx+i].w;"
            "var cy=y;"
            "for(var i=0;i<c.cy;++i)"
              "cy+=e.rows[i].h;"
            "var ch=0;"
            "for(var i=0;i<c.ch;++i)"
              "ch+=e.rows[c.cy+i].h;");
    out(    "%s(c,cx,cy,cw,ch);",element_set_geometry);
    out(  "});"
        "}");
}
#define get_table_set_geometry() genjs_get_function(sess,create_table_set_geometry)


void genjs_table_create(UpWidget* w,const char* name){
    Session* sess = w->session;
    UpTable* t = (UpTable*)w;
    const char* table_request_geometry = get_table_request_geometry();
    const char* table_set_geometry = get_table_set_geometry();
    out("%s={" GEOMETRY_REQ ":%s," GEOMETRY_SET ":%s,cols:[],rows:[]," CHILDREN ":[]," SPACING ":%d};",name,table_request_geometry,table_set_geometry,t->spacing);
    out("for(var i=0;i<%d;++i)"
          "%s.cols[i]=[];",t->cols,name);
    out("for(var i=0;i<%d;++i)"
          "%s.rows[i]=[];",t->rows,name);
}


void genjs_table_attach(UpWidget* parent, UpWidget* child){
    Session* sess = parent->session;
    const char* child_name = genjs_get_element(child);
    if(child->expand_width)
        out("%s." EXPAND_WIDTH "=1;",child_name);
    if(child->expand_width)
        out("%s." EXPAND_HEIGHT "=1;",child_name);
    out("%s.cx=%d;",child_name,child->cell_x);
    out("%s.cy=%d;",child_name,child->cell_y);
    out("%s.cw=%d;",child_name,child->cell_width);
    out("%s.ch=%d;",child_name,child->cell_height);
    out("%s." CHILDREN ".push(%s);",genjs_get_element(parent),child_name);
}

/* --- table widget ----------------------------------------------------- */

#if 0
typedef struct{
    UpWidget* child;
    int cell_x;
    int cell_y;
    int cell_width;
    int cell_height;
}UpTableItem;
#endif

static void up_table_done(UpWidget* w){
    UpTable* t = (UpTable*)w;
    queue_free_full(t->child_queue,(QueueNotifyFunc)u_ref_unref);
    up_widget_done_func(w);
}


static void up_table_link(UpWidget* w){
    UpTable* t = (UpTable*)w;
    List* iter;
    for(iter=t->child_queue->head;iter;iter=iter->next){
        UpWidget* child = (UpWidget*)iter->data;
        child->session = w->session;
        genjs_table_attach(w,child);
        up_widget_link(child);
    }
}


UpWidget* up_table_new(int rows,int cols,int homo){
    static const UpWidgetInterface up_table_vt = {
        .done = up_table_done,
        .genjs = genjs_table_create,
        .link = up_table_link
    };
    UpTable* t = u_new(UpTable,1);
    UpWidget* w = up_widget_init((UpWidget*)t);
    DEBUG_ref_reg((URef*)w,"Table");
    w->vt = &up_table_vt;
    t->cols = cols;
    t->rows = rows;
    t->child_queue = queue_new();
    t->spacing = 0;
    return w;
}


void up_table_attach(UpWidget* parent,UpWidget* child,int x1,int x2,int y1,int y2,int opt_w,int opt_h,int xspacing,int yspacing){
    child->expand_width  = opt_w&UP_EXPAND?1:0;
    child->expand_height = opt_h&UP_EXPAND?1:0;
    child->fill_width    = opt_w&UP_FILL?1:0;
    child->fill_height   = opt_h&UP_FILL?1:0;
    child->cell_x = x1;
    child->cell_y = y1;
    child->cell_width = x2-x1;
    child->cell_height = y2-y1;
    queue_append(((UpTable*)parent)->child_queue,child);
    child->parent = parent;
    if(parent->session){
        child->session = parent->session;
        genjs_table_attach(parent,child);
        up_widget_link(child);
    }
}
