/*
 * Copyright (C) 2016 Magnus Bergman
 *
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include "../genjs.h"
#include "../memutil.h"
#include "widget.h"

/* --- genjs -------------------------------------------------------- */

void genjs_create_element_request_geometry(Session* sess,const char* element_request_geometry){
    out("%s=function(e){",element_request_geometry);
    out(  "if(e." GEOMETRY_REQ "){"
            "e." GEOMETRY_REQ "(e);"
          "}else{"
            "e.w=e.mw;"
            "e.h=e.mh;"
            "if(!e." EXPAND_WIDTH ")"
              "e." EXPAND_WIDTH "=0;"
            "if(!e." EXPAND_HEIGHT ")"
              "e." EXPAND_HEIGHT "=0;"
          "}"
          "if(e.pw)"
            "e.w+=e.pw;"
          "if(e.ph)"
            "e.h+=e.ph;"
        "}");
}


void genjs_create_element_set_geometry(Session* sess,const char* element_set_geometry){
    out("%s=function(e,x,y,w,h){",element_set_geometry);
    out(  "var pw=e." FILL_WIDTH "||!e.mw?0:w-e.mw;"
          "x+=pw/2|0;"
          "w-=pw;"
          "var ph=e." FILL_HEIGHT "||!e.mh?0:h-e.mh;"
          "y+=ph/2|0;"
          "h-=ph;"
          "if(e.style){"
            "e.style.left=x+'px';"
            "e.style.top=y+'px';"
            "e.style.width=((e." GEOMETRY_SET "||e." EXPAND_WIDTH ")?w-e.pw:e.mw)+'px';"
            "e.style.height=((e." GEOMETRY_SET "||e." EXPAND_HEIGHT ")?h-e.ph:e.mh)+'px';"
          "}"
          "if(e." GEOMETRY_SET "){"
            "if(e.pw){"
              "x+=e.pw/2|0;"
              "w-=e.pw;"
            "}"
            "if(e.ph){"
              "y+=e.ph/2|0;"
              "h-=e.ph;"
            "}"
            "e." GEOMETRY_SET "(e,x,y,w,h);"
          "}"
        "}");
}


void genjs_create_element_init(Session* sess,const char* element_init){
    out("%s=function(e){",element_init);
    out(  "e.style.position='absolute';"
          "document.body.appendChild(e);"
          "e.mw=parseInt(getComputedStyle(e,null).width);"
          "e.mh=parseInt(getComputedStyle(e,null).height);"
          "e.pw=e.offsetWidth-e.mw;"
          "e.ph=e.offsetHeight-e.mh;"
        "}");
}

/* --- base widget -------------------------------------------------- */

static void up_widget_free_func(UpWidget* w){
    w->vt->done(w);
    u_free(w);
}


UpWidget* up_widget_init(UpWidget* w){
    w->ref.count = 1;
    w->ref.destroy = (URefDestroyFunc)up_widget_free_func;
    w->session = NULL;
    w->parent = NULL;
#if 0
    w->x = 0;
    w->y = 0;
    w->width = 0;
    w->height = 0;
#endif
    w->margin = 0;
    w->expand_width = 0;
    w->expand_height = 0;
    w->fill_width = 1;
    w->fill_height = 1;
    w->cell_x = 0;
    w->cell_y = 0;
    w->cell_width = 0;
    w->cell_height = 0;
    return w;
}


void up_widget_done_func(UpWidget* w){
}



#if 0
static void up_widget_shrink(UpWidget* w){
    int w2;
    int h2;
    int dummy;
    up_widget_request_geometry(w,&w2,&h2,&dummy,&dummy);
    w->x += (w->width-w2)/2;
    w->y += (w->height-h2)/2;
    w->width = w2;
    w->height = h2;
}
#endif

/*
static void up_widget_free(void* p){
    up_widget_done((UpWidget*)p);
    u_free(p);
}
*/
